# Algorithme Min Max et Monte Carlo

## Description du projet

Nous essayons ici d'implémenter les algorithmes de Monte Carlo et min max pour le jeu Ultimate Tic-tac-Toe afin de créer une IA intelligente. Le jeu Tic-tac-toe a été créé par les 
professeurs, il ne nous restait plus qu'à nous documenter sur ces deux algorithmes particulier afin de pouvoir les implémenter dans le cadre de notre projet.

## Liste des membres du projet

TRAN Alain -> travaille sur la branche MinMax_Alain sur l'algorithme Min-Max.  
CRESCENCE Cassandre -> travaille sur l'algorithme Monte Carlo.  
MACE Lloyd -> travaille sur l'algorithme Monte-Carlo.

## Consignes d'installation et d'exécution 

Le code est écrit en Java (sur la plateforme Netbeans). Pour pouvoir visualiser et tester le programme, il suffit néanmoins de posséder une ide capable de lire du Java (Netbeans,
Eclipse...)

### Pour exécuter l'algorithme de MCTS

Pour que le joueur ordinateur joue selon l'algorithme de MCTS, il faut modifier la classe `Player` par la classe `PlayerMCTS` (qui se trouve dans la branche `Master`> dossier `src` > dossier `tictactoecodingame` ).

On pourra modifier la classe, selon le type de partie que nous voulons lancer. Des indications en commentaire permettent de lancer la partie en 9x9 ou 3x3, avec l'IA de notre choix ( aléatoire ou MCTS)
    
### Pour exécuter l'algorithme MinMax

#### Pour le TicTacToe 3x3

Pour que le joueur ordinateur joue selon l'algorithme du MinMax dans le TicTacToe 3x3, il faut lancer la classe `Player_MinMax` ou modifier la classe `Player` comme ci-dessous :

    
    AlgoRechercheMinMax alea  = new AlgoRechercheMinMax(joueurOrdi, humain); 
    joueurOrdi.setAlgoRecherche(alea);
    
    GrilleTicTacToe3x3 grille = new GrilleTicTacToe3x3(); 

#### Pour le TicTacToe 9x9

Pour que le joueur ordinateur joue selon l'algorithme du MinMax dans le TicTacToe 9x9, il faut lancer la classe `Player_MinMax_9x9` ou modifier la classe `Player` comme ci-dessous :

    
    AlgoRechercheMinMax_9x9 alea  = new AlgoRechercheMinMax_9x9(joueurOrdi, humain); 
    joueurOrdi.setAlgoRecherche(alea);
    
    GrilleTicTacToe9x9 grille = new GrilleTicTacToe9x9(); 
    
    


## Porte d'entrée dans le code

### Algorithme MCTS

Pour accéder aux différentes implémentations de toutes les étapes du MCTS, il faut consulter la classe `MCTS` (qui se trouve dans la branche `Master` > dossier `src`> dossier `MCTS` ).
On y trouvera différents commentaires propres à chacunes des méthodes. 

Dans le dossier `MCTS` nous avons accès à différentes classes: Etat, noeud, UCT et Util ( qui contient une méthode permettant de changer le joueur sélectionné) contenant des constructeurs et méthodes nécessaires dans la mise en place du MCTS.

Dans la classe `PlayerMCTS` (qui se trouve dans la branche `Master`> dossier `src` > dossier `tictactoecodingame` ), nous pouvons:

-modifier le nombre d'itérations pour l'étape de simulation
-modifier le type de partie qu'on souhaite lancer

### Algorithme MinMax

Pour accéder aux implémentations du MinMax, il faut consulter le dossier `src` puis dans le dossier `tictactoecodeingame`.

Les deux classes qui implémentent le MinMax sont `AlgoRechercheMinMax` et `AlgoRechercheMinMax_9x9` pour respectivement le TicTacToe 3x3 et le TicTacToe_9x9.

Dans la classe `AlgoRechercheMinMax_9x9`, nous pouvons améliorer l'heuristique en modifiant la profondeur max : plus elle est grande plus elle sera précise mais plus l'algorithme
sera long.


## Les pistes d'améliorations 

### Pour l'algorithme MCTS 

Comme première piste d'amélioration du temps d'exécution de l'algorithme, nous pouvons utiliser une technique heuristique améliorant l'étape de simulation. En effet, au lieu de lancer des simulations aléatoires, nous pouvons lancer des simulations qui seront optimisées à partir des simulations précédentes.
Une technique connue appelé RAVE ( Rapid action value estimation) permet d'optimiser l'étape de simulations. Pour l'implémenter, il faut ajouter de nouveaux paramètres qui considèrent la statistique des victoires associée  à chaque noeud en prenant également en compte le moment où le coup associé a été joué.
Ces paramètres permettront de réécrire la formule de l'UCT (Upper Confidence Bound 1 applied to Trees) sous une autre forme qui prendra en compte ces nouveaux critères statistiques.

### Pour l'algorithme Minmax

Pour l'algorithme de Minmax, nous pouvons élaborer la technique d'élagage Alpha-bêta qui va permettre de réduire le nombre de noeud à visiter. En effet, celui-ci va par exemple
pour la phase de maximisation vérifier ses différents fils et s'il voit qu'une de ses fils aura au maximum une valeur inférieure à la valeur d'un fils déjà visité avec la
fonction MinMax et d'évaluation alors il ne prendra pas la peine de visiter cette branche fille. De même pour la minimisation.
Cette méthode peut permettre de réduire considérablement la complexité de l'algorithme en particulier pour le TicTacToe 9x9 qui peut être très long en fonction de l'heuristique
choisie.